==> grown-0.4.0.tgz: package/lib/plugs/mailer.js <==
'use strict';

const debug = require('debug')('grown:mailer');

const _tpl = require('../util/_tpl');

const glob = require('glob');
const path = require('path');

module.exports = args => {
  const _folders = [];
  const _handlers = {};
  const _cachedPaths = {};

  ((!Array.isArray(args) && args ? [args] : args) || []).forEach(opts => {
    ((!Array.isArray(opts.folders) && opts.folders ? [opts.folders] : opts.folders) || []).forEach(cwd => {
      debug('Loading mailer views from %s', cwd);

      _folders.push(cwd);
    });

    // scan handlers
    ((!Array.isArray(opts.handlers) && opts.handlers ? [opts.handlers] : opts.handlers) || []).forEach(cwd => {
      debug('Loading handlers from %s', cwd);

      glob.sync('**/*.js', { cwd, nodir: true }).forEach(src => {
        debug('Registering mailer from %s', path.join(cwd, src));

        _handlers[src.replace(/(?:index)?\.js$/, '').replace(/\//g, '.')] = {
          filepath: path.join(cwd, src),
        };
      });
    });
  });

  function _render(tpl, util, _context) {
    const _id = tpl.src;

    try {
      let _fn;

      if (typeof _id === 'function') {
        debug('Rendering function <%s>', _id.name || '?');

        _fn = _id;
      } else {
        debug('Rendering view <mailers/%s>', _id);

        /* istanbul ignore else */
        if (!_cachedPaths[_id]) {
          _cachedPaths[_id] = _tpl.find(`mailers/${_id}`, _folders);
        }
      }

      return util.props(tpl.data, v => (typeof v === 'function' ? v(_context || {}) : v))
        .then(locals => {
          /* istanbul ignore else */
          if (!_fn && _cachedPaths[_id].indexOf('.js') === -1) {
            throw new Error(`Cannot use '${_id}' as template`);
          }

          return _tpl.run(_fn || require(_cachedPaths[_id]), locals);
        });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  return $ => {
    // real transport
    function _proxy(delegate, _context) {
      /* istanbul ignore else */
      if (delegate && typeof delegate !== 'function') {
        throw new Error(`Invalid transport, given: '${delegate}'`);
      }

      /* istanbul ignore else */
      if (typeof delegate === 'function') {
        return delegate(_context);
      }

      // fallback
      return {
        sendMail(opts, cb) {
          cb(undefined, opts);
        },
      };
    }

    function _view(tpl, _context) {
      const _call = 'send';
      const _vars = {};

      // check mail
      let _sent;

      return Promise.resolve()
        .then(() => {
          /* istanbul ignore else */
          if (!_handlers[tpl]) {
            throw new Error(`Missing '${tpl}' mailer`);
          }

          const Mailer = $.util.load($.extensions.mailers, _handlers[tpl], tpl);
          const mailerInstance = _handlers[tpl].instance;

          /* istanbul ignore else */
          if (!mailerInstance[_call]) {
            throw new Error(`Missing '${_call}' for ${tpl} mailer`);
          }

          // prepare mail locals
          mailerInstance[_call].call(_vars, (to, subject, body) => {
            $.util.extend(_vars, { to, subject, body },
              mailerInstance.defaults,
              Mailer.defaults);

            // set default layout
            _vars.layout = _vars.layout
              || mailerInstance.layout
              || Mailer.layout;

            // set default transport
            _vars.transport = _vars.transpot
              || mailerInstance.transport
              || Mailer.transport;

            _sent = true;
          }, _context);
        })
        .then(() => {
          /* istanbul ignore else */
          if (!_sent) {
            throw new Error(`Missing call mail() on '${tpl}.${_call}'`);
          }

          return _vars;
        });
    }

    function _send(tpl, locals, _transport) {
      const _context = {};

      Object.keys($.extensions).forEach(key => {
        _context[key] = $.extensions[key];
      });

      /* istanbul ignore else */
      if (typeof tpl === 'object') {
        locals = $.util.extend({}, tpl, locals);
        tpl = locals.layout || 'default';
      }

      return Promise.resolve()
        .then(() => (typeof tpl === 'string' ? _view(tpl, _context) : locals))
        .then(_locals => {
          _transport = _locals.transport || _transport;

          tpl = _locals.layout || tpl;

          delete _locals.transport;
          delete _locals.layout;

          // FIXME: views/mailers/default === views/{path}/{tpl}
          // FIXME: how implement plain text support?
          // FIXME: how to preview and test mailers?

          // FIXME:
          //
          //  .mail [TO] [SUBJECT] [BODY] [local=value] [foo=bar] [...]
          //    --preview|-p
          //    --mailer|-m
          //    --send|-s -cco|-o --cc|-c
          //    --to|-t --subject|-u --body|-b --attach|-a
          //
          //  .mail [default] --preview
          //  .mail pateketrueke@gmail.com "Hello" "WORLD"
          //  .mail user.activate email=foo@candy.bar code=123 --send
          //  .mail "my subject" `JSON.stringify(process.env, null, 2)` --mailer external
          //

          // pass extensions as static context
          return _render({ src: tpl, data: _locals }, $.util, _context)
            .then(result => new Promise((resolve, reject) =>
              _proxy(locals.transport || _transport, _context)
              .sendMail({
                subject: _locals.subject || locals.subject,
                from: _locals.from || locals.from,
                to: _locals.to || locals.to,
                html: result,
              }, (err, info) => {
                if (err) {
                  reject({
                    status: 'error',
                    result: err.message || err.toString(),
                  });
                } else {
                  resolve({
                    status: 'sent',
                    result: info,
                  });
                }
              })))
            .catch(err => ({
              status: 'error',
              result: err.message || err.toString(),
            }));
        });
    }

    $.extensions.mailers = $.extensions();

    // static mail sender
    $.extensions.mail = _send;

    $.extensions('Conn', {
      methods: {
        mail: _send,
      },
    });
  };
};

==> grown-0.4.4.tgz: package/lib/plugs/mailer.js <==
'use strict';

const debug = require('debug')('grown:mailer');

const _tpl = require('../util/_tpl');

const glob = require('glob');
const path = require('path');

module.exports = args => {
  const _folders = [];
  const _handlers = {};
  const _cachedPaths = {};

  ((!Array.isArray(args) && args ? [args] : args) || []).forEach(opts => {
    ((!Array.isArray(opts.folders) && opts.folders ? [opts.folders] : opts.folders) || []).forEach(cwd => {
      debug('Loading mailer views from %s', path.relative(process.cwd(), cwd));

      _folders.push(cwd);
    });

    // scan handlers
    ((!Array.isArray(opts.handlers) && opts.handlers ? [opts.handlers] : opts.handlers) || []).forEach(cwd => {
      debug('Loading handlers from %s', path.relative(process.cwd(), cwd));

      glob.sync('**/*.js', { cwd, nodir: true }).forEach(src => {
        debug('Registering mailer from %s', path.relative(process.cwd(), path.join(cwd, src)));

        _handlers[src.replace(/(?:index)?\.js$/, '').replace(/\//g, '.')] = {
          filepath: path.join(cwd, src),
        };
      });
    });
  });

  function _render(tpl, util, _context) {
    const _id = tpl.src;

    try {
      let _fn;

      if (typeof _id === 'function') {
        debug('Rendering function <%s>', _id.name || '?');

        _fn = _id;
      } else {
        debug('Rendering view <mailers/%s>', _id);

        /* istanbul ignore else */
        if (!_cachedPaths[_id]) {
          _cachedPaths[_id] = _tpl.find(`mailers/${_id}`, _folders);
        }
      }

      return util.props(tpl.data, v => (typeof v === 'function' ? v(_context || {}) : v))
        .then(locals => {
          /* istanbul ignore else */
          if (!_fn && _cachedPaths[_id].indexOf('.js') === -1) {
            throw new Error(`Cannot use '${_id}' as template`);
          }

          return _tpl.run(_fn || require(_cachedPaths[_id]), locals);
        });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  return $ => {
    // real transport
    function _proxy(delegate, _context) {
      /* istanbul ignore else */
      if (delegate && typeof delegate !== 'function') {
        throw new Error(`Invalid transport, given: '${delegate}'`);
      }

      /* istanbul ignore else */
      if (typeof delegate === 'function') {
        return delegate(_context);
      }

      // fallback
      return {
        sendMail(opts, cb) {
          cb(undefined, opts);
        },
      };
    }

    function _view(tpl, _context) {
      const _call = 'send';
      const _vars = {};

      // check mail
      let _sent;

      return Promise.resolve()
        .then(() => {
          /* istanbul ignore else */
          if (!_handlers[tpl]) {
            throw new Error(`Missing '${tpl}' mailer`);
          }

          const Mailer = $.util.load($.extensions.mailers, _handlers[tpl], tpl);
          const mailerInstance = _handlers[tpl].instance;

          /* istanbul ignore else */
          if (!mailerInstance[_call]) {
            throw new Error(`Missing '${_call}' for ${tpl} mailer`);
          }

          // prepare mail locals
          mailerInstance[_call].call(_vars, (to, subject, body) => {
            $.util.extend(_vars, { to, subject, body },
              mailerInstance.defaults,
              Mailer.defaults);

            // set default layout
            _vars.layout = _vars.layout
              || mailerInstance.layout
              || Mailer.layout;

            // set default transport
            _vars.transport = _vars.transpot
              || mailerInstance.transport
              || Mailer.transport;

            _sent = true;
          }, _context);
        })
        .then(() => {
          /* istanbul ignore else */
          if (!_sent) {
            throw new Error(`Missing call mail() on '${tpl}.${_call}'`);
          }

          return _vars;
        });
    }

    function _send(tpl, locals, _transport) {
      const _context = {};

      Object.keys($.extensions).forEach(key => {
        _context[key] = $.extensions[key];
      });

      /* istanbul ignore else */
      if (typeof tpl === 'object') {
        locals = $.util.extend({}, tpl, locals);
        tpl = locals.layout || 'default';
      }

      return Promise.resolve()
        .then(() => (typeof tpl === 'string' ? _view(tpl, _context) : locals))
        .then(_locals => {
          _transport = _locals.transport || _transport;

          tpl = _locals.layout || tpl;

          delete _locals.transport;
          delete _locals.layout;

          // FIXME: views/mailers/default === views/{path}/{tpl}
          // FIXME: how implement plain text support?
          // FIXME: how to preview and test mailers?

          // FIXME:
          //
          //  .mail [TO] [SUBJECT] [BODY] [local=value] [foo=bar] [...]
          //    --preview|-p
          //    --mailer|-m
          //    --send|-s -cco|-o --cc|-c
          //    --to|-t --subject|-u --body|-b --attach|-a
          //
          //  .mail [default] --preview
          //  .mail pateketrueke@gmail.com "Hello" "WORLD"
          //  .mail user.activate email=foo@candy.bar code=123 --send
          //  .mail "my subject" `JSON.stringify(process.env, null, 2)` --mailer external
          //

          // pass extensions as static context
          return _render({ src: tpl, data: _locals }, $.util, _context)
            .then(result => new Promise((resolve, reject) =>
              _proxy(locals.transport || _transport, _context)
              .sendMail({
                subject: _locals.subject || locals.subject,
                from: _locals.from || locals.from,
                to: _locals.to || locals.to,
                html: result,
              }, (err, info) => {
                if (err) {
                  reject({
                    status: 'error',
                    result: err.message || err.toString(),
                  });
                } else {
                  resolve({
                    status: 'sent',
                    result: info,
                  });
                }
              })))
            .catch(err => ({
              status: 'error',
              result: err.message || err.toString(),
            }));
        });
    }

    $.extensions.mailers = $.extensions();

    // static mail sender
    $.extensions.mail = _send;

    $.extensions('Conn', {
      methods: {
        mail: _send,
      },
    });
  };
};
